function HX = HX_up_inc(    hotst   , hstfrac , Thout , Thin  , ...
                            coldst  , cstfrac , Tcin  , Tcout , ...
                            maxduty , dutyinc , dTmin , plot         )
% Calculates HX from T vs. q stream curves
% TODO: review Arguments&Returns
% Arguments: hotst,coldst   = streams matrix, 1st column T [K], 2nd column qs [e.g.,kW]
%                             starting at q=0 and q-sorted; qs > 0: cold , qs < 0: hot .
%                             If just two rows stream is assumed constant mCp.
%            hst,cstfrac    = flowrate fraction of stream involved in this HX. []
%            T[h|c][in|out] = Hot|Cold,inlet|outlet temperatures.  -1 if unknow.
%                             If cold side Ts != -1 and hot side Ts == -1 , the maximun
%                             duty up-in-T is calculated until dT == dTmin or one
%                             stream is exhausted; and viceversa. [K]
%            maxduty        = same units that qs. -1 if unknow. if != -1 the calculation
%                             proceeds till max. duty is completed or dTmin is found. [kW]
%            dutyinc        = step increment for duty. If > 0 calculation starts at cold
%                             end and proceeds upwards and viceversa.
%            dTmin          = Minimun allowed T approach between streams. [K]
%            plot           = flag if != 0, plot HX T vs. duty. Else, don't.
% Returns  : HX = matrix containing ... 
%            Thout   Thin   = Hot outlet , Hot inlet Ts   [K]
%            Tcin    Tcout  = Cold inlet , Cold outlet Ts [K]
%            duty    dTln   = Duty [kW]  , logarithmic mean of extreme T differences [K]
%            restHc  restHh = Entalpy flows to finish cold and hot stream needs,
%                             beyond this HX, in the same direction of calculation [kW]
%            cstfrac hstfrac= cold, hot stream fraction involved in this HX. []
%            coldT   hotdT  = T differences in the cold, hot side [K]
%            mindT   maxdT  = Minimum and maximum T difference [K]

% Copyright (c) Fidel Mato & UVa 2018
% Author    : Fidel Mato <fidel@iq.uva.es>
% Maintainer: Fidel Mato <fidel@iq.uva.es>
% Keywords  : Heat integration, Heat exchanger

% [Note: kW are used as an alias for any thermal flow unit.]

%nsteps   = 200000 ;
%nsteps   =  100000 ;

Ths      = hotst( : , 1 )          ;                       % [K]
Hhs      = hotst( : , 2 )*hstfrac  ;                       % [kW]
Tcs      = coldst( : , 1 )         ;                       % [K]
Hcs      = coldst( : , 2 )*cstfrac ;                       % [kW]

if( dutyinc > 0 )

    if( isnan( Thout ) || isnan( Tcin ) )
        error( 'HX_up_inc(...) : incoherent Thout or Tcin and dutyinc' ) ;
        return ;
    end
    if( Thout < Ths( end ) || Tcin < Tcs( 1 ) )
        error( 'HX_up_inc(...) : Thout or Tcin out of range (too low)' ) ;
        return ;
    end
    if( Thout > Ths( 1 ) || Tcin > Tcs( end ) )
        error( 'HX_up_inc(...) : Thout or Tcin out of range (too high)' ) ;
        return ;
    end

    if( ~isnan( Thin ) )
        if( Thin >= Ths( 1 ) || Thin < Thout )
            Hhlim = Hhs( 1 )                     ;         % [kW]
        else
            Hhlim = interp1( Ths , Hhs , Thin  ) ;         % [kW]
        end
    else
        Hhlim = Hhs( 1 )                         ;         % [kW]
    end  
    if( ~isnan( Tcout ) )
        if( Tcout >= Tcs( end ) || Tcout < Tcin )
            Hclim = Hcs( end )                   ;         % [kW]
        else
            Hclim = interp1( Tcs , Hcs , Tcout ) ;         % [kW]
        end
    else
        Hclim = Hcs( end )                       ;         % [kW]
    end  
    Hhout    = interp1( Ths , Hhs , Thout ) ;                % [kW]
    Hcin     = interp1( Tcs , Hcs , Tcin  ) ;                % [kW]

    if( Hhlim - Hhout >= Hclim - Hcin )
        H1 = Hcin  ;  H2 = Hclim ;
    else
        H1 = Hhout ;  H2 = Hhlim ;
    end
    Hsteps = [ H1 : dutyinc : H2 ] .- H1 ;
    if( rem( H2 , dutyinc ) ~= 0 )
        Hsteps = [ Hsteps , ( H2 - H1 ) ] ;
    end
    if( ~isnan( maxduty ) && ( maxduty < ( H2 - H1 ) ) )
        Hsteps = Hsteps( Hsteps < maxduty ) ;
        Hsteps = [ Hsteps , maxduty ]       ;
    end  
    
    duties = []    ;
    Tcolds = []    ;
    Thots  = []    ;
    mindT  = 1e3   ;
    maxdT  = 0     ;
    for( duty = Hsteps )
        Th   = interp1( Hhs , Ths , Hhout + duty ) ;       % [K]
        Tc   = interp1( Hcs , Tcs , Hcin + duty )  ;       % [K]
        dT   = Th - Tc                             ;       % [K]
        %fprintf( '%8.4f %8.4f %10.5f %6.4f\n' , Th , Tc , duty , dT ); % DEBUG
        if( dT < dTmin && abs( dT - dTmin ) > 1e-8 )
            break ;
        end    
        duties = [duties ; duty] ; Tcolds = [Tcolds ; Tc] ; Thots = [Thots ; Th] ;
        if( dT < mindT ) mindT = dT ; end
        if( dT > maxdT ) maxdT = dT ; end
    end
    Thin   = Thots( end )             ;                    % [K]
    Tcout  = Tcolds( end )            ;                    % [K]
    duty   = duties( end )            ;                    % [kW]
    restHh = Hhlim - ( Hhout + duty ) ;                    % [kW]
    restHc = Hclim - ( Hcin + duty )  ;                    % [kW]

%elseif( Thout == -1 && Tcin == -1 )
%
%    Hhin     = interp1( Ths , Hhs , Thin )  ;                % [kW]
%    Hhlim    = Hhs( end )                   ;                % [kW]
%    Hcout    = interp1( Tcs , Hcs , Tcout ) ;                % [kW]
%    Hclim    = Hcs( 1 )                     ;                % [kW]
%
%    if( Hhin - Hhlim >= Hcout - Hclim )
%        H1 = Hcout  ;  H2 = Hclim ; incH = ( Hclim - Hcout )/nsteps ;
%    else
%        H1 = Hhin ;  H2 = Hhlim ; incH = -( Hhin - Hhlim )/nsteps ;
%    end
%
%    duties = []    ;
%    Tcolds = []    ;
%    Thots  = []    ;
%    mindT  = 1e3   ;
%    maxdT  = 0     ;
%    for( H = H1 : incH : H2 )
%        duty = H - H1                              ;       % [kW]
%        Th   = interp1( Hhs , Ths , Hhin + duty )  ;       % [K]
%        Tc   = interp1( Hcs , Tcs , Hcout + duty ) ;       % [K]
%        dT   = Th - Tc                             ;       % [K]
%        %fprintf( '%8.3f %8.3f %10.2f %6.2f\n' , Th , Tc , duty , dT ); % DEBUG
%        if( dT < dTmin && abs( dT - dTmin ) > 1e-8 )
%            break ;
%        end    
%        duties = [duties ; duty] ; Tcolds = [Tcolds ; Tc] ; Thots = [Thots ; Th] ;
%        if( dT < mindT ) mindT = dT ; end
%        if( dT > maxdT ) maxdT = dT ; end
%    end
%    Thout  = Thots( end )             ;                    % [K]
%    Tcin   = Tcolds( end )            ;                    % [K]
%    restHh = Hhlim - ( Hhin + duty )  ;                    % [kW]
%    restHc = Hclim - ( Hcout + duty ) ;                    % [kW]

end

if( plot ~= 0 )
    clf ;
    line( duties , Thots  , 'LineStyle' , '-' , 'linewidth' , 1.5 , 'Marker' , 'none' ,...
                            'Color' , 'r' ) ;
    line( duties , Tcolds , 'LineStyle' , '-' , 'linewidth' , 1.5 , 'Marker' , 'none' ,...
                            'Color' , 'b' ) ;
end
% ------------------------------------------------------------------------------ Returning
coldT    = Thout - Tcin ;                                  % [K]
hotdT    = Thin - Tcout ;                                  % [K]
if( coldT ~= hotdT )
    dTln     = ( coldT - hotdT )/log( coldT/hotdT ) ;      % [K]
else
    dTln =coldT ;                                          % [K]
end
HX       = ...
  [ Thout   Thin
    Tcin    Tcout
    duty    dTln
    restHc  restHh
    cstfrac hstfrac
    coldT   hotdT
    mindT   maxdT  ] ;

return ;

end
% -------------------------------------------------------------------------------- __END__
