function dSid_CO = AL_dSid_CO( T1 , T2 )
%
% AL_dSid_CO - Carbon monoxide ideal entropy difference using Aly&Lee equation.
%
%   Aly, F.A. and Lee, L.L., Self-Consistent Equations for Calculating the Ideal Gas
%   Heat Capacity, Enthalpy and Entropy, Fluid Phase Equilibria, 6 (1981) pp 169-179.
%
%   call  : AL_dSid_CO( T1 , T2 ) where [ T1, T2 ] = [ K , K ]
%   return: dSid_CO in [ J/mol/K ]

% ---------------------------------------------------------- CO (273.15-1500 K) Parameters
% From Aly&Lee_1981 Table 1, p 174

% B_CO  =     6.95854*4.1840                                 % [cal/mol/K => J/mol/K]
% C_CO  =     2.02441*4.1840                                 % [cal/mol/K => J/mol/K]
% D_CO  =  1541.22     ;                                     % [K]
% E_CO  =     0.096774*4.1840                                % [cal/mol/K => J/mol/K]
% F_CO  =  3674.81     ;                                     % [K]
% G_CO  =     6.23387*4.1840                                 % [cal/mol/K => J/mol/K]

B_CO  =    29.11453136  ;                                  % [J/mol/K]
C_CO  =     8.47013144  ;                                  % [J/mol/K]
D_CO  =  1541.22        ;                                  % [K]
E_CO  =     0.404902416 ;                                  % [J/mol/K]
F_CO  =  3674.81        ;                                  % [K]
G_CO  =    26.08251208  ;                                  % [J/mol/K]

B = B_CO  ; C = C_CO  ; D = D_CO  ; E = E_CO  ; F = F_CO  ; G = G_CO  ;

% ---------------------------------------------------------------------------- Calculation
T = T2 ;                                                   % [K]
dSid_CO = B*log( T ) + C*( D/T*coth( D/T ) - log( sinh( D/T ) ) ) - ...
            E*( F/T*tanh( F/T ) - log( cosh( F/T ) ) ) ;                % [J/mol/K]

T = T1 ;                                                   % [K]
dSid_CO =  dSid_CO - ( B*log( T ) + C*( D/T*coth( D/T ) - log( sinh( D/T ) ) ) - ...
            E*( F/T*tanh( F/T ) - log( cosh( F/T ) ) ) ) ;              % [J/mol/K]

return ;
