function dSid_CO2 = AL_dSid_CO2( T1 , T2 )
%
% AL_dSid_CO2 - Carbon dioxide ideal entropy difference using Aly&Lee equation.
%
%   Aly, F.A. and Lee, L.L., Self-Consistent Equations for Calculating the Ideal Gas
%   Heat Capacity, Enthalpy and Entropy, Fluid Phase Equilibria, 6 (1981) pp 169-179.
%
%   call  : AL_dSid_CO2( T1 , T2 ) where [ T1, T2 ] = [ K , K ]
%   return: dSid_CO2 in [ J/mol/K ]

% --------------------------------------------------------- CO2 (273.15-1500 K) Parameters
% From Aly&Lee_1981 Table 1, p 174

% B_CO2  =    7.54056*4.1840                                       % [cal/mol/K => J/mol/K]
% C_CO2  =    7.51625*4.1840                                       % [cal/mol/K => J/mol/K]
% D_CO2  = 1442.70     ;                                     % [K]
% E_CO2  =    5.38023*4.1840                                       % [cal/mol/K => J/mol/K]
% F_CO2  =  647.502    ;                                     % [K]
% G_CO2  =    6.25230*4.1840                                       % [cal/mol/K => J/mol/K]

B_CO2  =   31.54970304 ;                                   % [J/mol/K]
C_CO2  =   31.44799000 ;                                   % [J/mol/K]
D_CO2  = 1442.70       ;                                   % [K]
E_CO2  =   22.51088232 ;                                   % [J/mol/K]
F_CO2  =  647.502      ;                                   % [K]
G_CO2  =   26.1596232  ;                                   % [J/mol/K]

B = B_CO2  ; C = C_CO2  ; D = D_CO2  ; E = E_CO2  ; F = F_CO2  ; G = G_CO2  ;

% ---------------------------------------------------------------------------- Calculation
T = T2 ;                                                   % [K]
dSid_CO2 = B*log( T ) + C*( D/T*coth( D/T ) - log( sinh( D/T ) ) ) - ...
            E*( F/T*tanh( F/T ) - log( cosh( F/T ) ) ) ;                % [J/mol/K]

T = T1 ;                                                   % [K]
dSid_CO2 =  dSid_CO2 - ( B*log( T ) + C*( D/T*coth( D/T ) - log( sinh( D/T ) ) ) - ...
            E*( F/T*tanh( F/T ) - log( cosh( F/T ) ) ) ) ;              % [J/mol/K]

return ;
