function dSid_H2O = AL_dSid_H2O( T1 , T2 )
%
% AL_dSid_H2O - Oxygen ideal entropy difference using Aly&Lee equation.
%
%   Aly, F.A. and Lee, L.L., Self-Consistent Equations for Calculating the Ideal Gas
%   Heat Capacity, Enthalpy and Entropy, Fluid Phase Equilibria, 6 (1981) pp 169-179.
%
%   call  : AL_dSid_H2O( T1 , T2 ) where [ T1, T2 ] = [ K , K ]
%   return: dSid_H2O in [ J/mol/K ]

% --------------------------------------------------------- H2O (273.15-1500 K) Parameters
% From Aly&Lee_1981 Table 1, p 174
% B_H2O  =    7.97183*4.1840                                 % [cal/mol/K => J/mol/K]
% C_H2O  =    6.27078*4.1840                                 % [cal/mol/K => J/mol/K]
% D_H2O  = 2572.63     ;                                     % [K]
% E_H2O  =    2.05010*4.1840                                 % [cal/mol/K => J/mol/K]
% F_H2O  = 1156.72     ;                                     % [K]
% G_H2O  =   -3.24989*4.1840                                 % [cal/mol/K => J/mol/K]

B_H2O  =   33.35413672 ;                                     % [J/mol/K]
C_H2O  =   26.23694352 ;                                     % [J/mol/K]
D_H2O  = 2572.63       ;                                     % [K]
E_H2O  =    8.57761840 ;                                     % [J/mol/K]
F_H2O  = 1156.72       ;                                     % [K]
G_H2O  =  -13.59753976 ;                                     % [J/mol/K]

B = B_H2O  ; C = C_H2O  ; D = D_H2O  ; E = E_H2O  ; F = F_H2O  ; G = G_H2O  ;

% ---------------------------------------------------------------------------- Calculation
T = T2 ;                                                   % [K]
dSid_H2O = B*log( T ) + C*( D/T*coth( D/T ) - log( sinh( D/T ) ) ) - ...
            E*( F/T*tanh( F/T ) - log( cosh( F/T ) ) ) ;                % [J/mol/K]

T = T1 ;                                                   % [K]
dSid_H2O =  dSid_H2O - ( B*log( T ) + C*( D/T*coth( D/T ) - log( sinh( D/T ) ) ) - ...
            E*( F/T*tanh( F/T ) - log( cosh( F/T ) ) ) ) ;              % [J/mol/K]

return ;
