function dSid_N2 = AL_dSid_N2( T1 , T2 )
%
% AL_dSid_N2 - Nitrogen ideal entropy difference using Aly&Lee equation.
%
%   Aly, F.A. and Lee, L.L., Self-Consistent Equations for Calculating the Ideal Gas
%   Heat Capacity, Enthalpy and Entropy, Fluid Phase Equilibria, 6 (1981) pp 169-179.
%
%   call  : AL_dSid_N2( T1 , T2 ) where [ T1, T2 ] = [ K , K ]
%   return: dSid_N2 in [ J/mol/K ]

% ---------------------------------------------------------- N2 (273.15-1500 K) Parameters
% From Aly&Lee_1981 Table 1, p 174
% B_N2  =     6.95808*4.1840  ;                              % [cal/mol/K => J/mol/K]
% C_N2  =     2.03952*4.1840  ;                              % [cal/mol/K => J/mol/K]
% D_N2  =  1681.60            ;                              % [K]
% E_N2  =     0.506863*4.1840 ;                              % [cal/mol/K => J/mol/K]
% F_N2  =  6535.66            ;                              % [K]
% G_N2  =     5.05495*4.1840  ;                              % [cal/mol/K => J/mol/K]

B_N2  =    29.11260672  ;                                  % [J/mol/K]
C_N2  =     8.53335168  ;                                  % [J/mol/K]
D_N2  =  2522.05        ;                                  % [K]
E_N2  =     2.120714792 ;                                  % [J/mol/K]
F_N2  =  6535.66        ;                                  % [K]
G_N2  =    21.1499108   ;                                  % [J/mol/K]

B = B_N2  ; C = C_N2  ; D = D_N2  ; E = E_N2  ; F = F_N2  ; G = G_N2  ;

% ---------------------------------------------------------------------------- Calculation
T = T2 ;                                                   % [K]
dSid_N2 = B*log( T ) + C*( D/T*coth( D/T ) - log( sinh( D/T ) ) ) - ...
            E*( F/T*tanh( F/T ) - log( cosh( F/T ) ) ) ;                % [J/mol/K]

T = T1 ;                                                   % [K]
dSid_N2 =  dSid_N2 - ( B*log( T ) + C*( D/T*coth( D/T ) - log( sinh( D/T ) ) ) - ...
            E*( F/T*tanh( F/T ) - log( cosh( F/T ) ) ) ) ;              % [J/mol/K]

return ;
