function split = Constant_Kis_LV_FLASH( zis , Kis , strict )
% Constant_Kis_LV_FLASH --  Calculates Liquid/Vapor phase splitting of a constant K values
%                           mixture using Michelsen approach to Rachford-Rice equation.
% Calling : zis    = global mixture composition (molar fractions) vector
%           Kis    = K values vector
%           strict = (1/0) flag :
%                    1 : returns strictly [ 0 <= vapor fraction <= 1 ], or an error code
%                    0 : allows returning vapor fractions outside 0 <= V <= 1 (iteration) 
%
% Returns : split = matrix containing vapor fraction and phase compositions:
%           split = [ bV   0    0    ...  0           [mol/mol]  vapor fraction
%                     x1   x2   x3   ...  xn          [mol.frac] liquid phase
%                     y1   y2   y3   ...  yn  ]       [mol.frac] vapor phase
%
% Errors   : returning (-2) : (strict == 1) out of bounds, not a solution between 0 and 1
%            returning (-1) : composition sum check failed for liquid or vapor

% Copyright (c) Fidel Mato & UVa 2017, 2018
% Author    : Fidel Mato <fidel@iq.uva.es>
% Keywords  : Flash , Ideal , Two phase , K values , Rachford-Rice , Michelsen

% ----------------------------------------- Constant Kis FLASH
% Following the procedure from Michelsen , pg 253

% i.  Check conditions (4) and (5) for the existence of a solution:
%     a solution ( V = 0 ... 1 ) exists if g(0) > 0 , g(1) < 0 :
g0       = sum( zis.*Kis ) - 1  ;          % == sum( zis.*( Kis  - 1 ) )  ;
g1       = 1 - sum( zis./Kis )  ;
if( ~( g0 > 0 && g1 < 0 ) && strict ~= 0 )
    split = -2 ;
    return ;
end

ncomp    = length( zis )         ;                         % [] n compounds

if( ncomp == 2 )                                           % binary, explicit

    V   = zis( 1 )/( 1 - Kis( 2 ) ) + zis( 2 )/( 1 - Kis( 1 ) ) ; % [mol/mol]
    xis = zis./( 1 + V*( Kis - 1 ) ) ;                            % [mol.frac]
    yis = xis.*Kis                   ;                            % [mol.frac]
    split = [ V zeros( ncomp - 1 ) ; xis ; yis ] ;
    return ;

end

% ii. Test conditions (7) and (8) for whether tighter bounds can be set
Vmin     = 0 ;
Vmax     = 1 ;
Vmins    = [ ] ;   
Vmaxs    = [ ] ;   
for( k = 1 : ncomp )
    Ki = Kis( k ) ;
    zi = zis( k ) ;
    if( Ki >= 1 )
        if( ( Ki*zi - 1 )/( Ki - 1 ) > 0 )
            Vmins = [ Vmins  ( Ki*zi - 1 )/( Ki - 1 ) ] ;
            Vmin = max( Vmins ) ;
        end
    else
        if( ( 1 - zi )/( 1 - Ki ) > 0 )
            Vmaxs = [ Vmaxs  ( 1 - zi )/( 1 - Ki ) ] ;
            Vmax = min( Vmaxs ) ;
        end
    end
end

if( Vmin < 0 )
    Vmin = 0 ;
end
if( Vmax > 1 ) % ????
    Vmax = 1 ;
end

% iii. Calculate Beta_ini ...
V = ( Vmin + Vmax )/2    ;

if( sum( zis.*( Kis - 1 )./( 1 + V*( Kis - 1 ) ) ) > 0 )   % ??????????????
    Vmin = V ;         % ... If g(Vini) > 0 , set Vmin = Vini
else
    Vmax = V ;         % otherwise set Vmax = Vini
    % TODO: here, check if shifting from betaV to betaL
    % as iterating variable is advisable [ g(0.5)... ].
end

maxiters = 100   ;
tol      = 1e-10 ;
for( k = 1 : maxiters )
    
    % iv.  Calculate g(V) and dg(V)/dV ...
    gV  = sum( zis.*( Kis - 1 )./( 1 + V*( Kis - 1 ) ) )        ;
    dg = -sum( zis.*( Kis - 1 ).^2./( 1 + V*( Kis - 1 ) ).^2 )  ;

    if( gV > 0 )
        Vmin = V ;         % ... If g(V) > 0 , set Vmin = V
    else
        Vmax = V ;         % otherwise set Vmax = V
    end
    
    % ... calculate the Newton step dV and set Vnew = V + dV
    step = - gV/dg  ;
    V    = V + step ;
    
    % v.  If Vmin < Vnew < Vmax ...
    if( ( Vmin < V ) && ( V < Vmax ) )
       V = V ;                 % ... set V = Vnew
    else
       V = ( Vmin + Vmax )/2 ; % ... otherwise ...
    end
    
    % vi.  If the length of the Newton step is above the tolerance set for the solution...
    %      return to iv. Otherwise we are done
    if( abs( step ) < tol )
        break ; 
    end
    
end
xis      = zis./( 1 + V*( Kis - 1 ) ) ;                    % [mol.frac]
yis      = xis.*Kis                   ;                    % [mol.frac]

%%%%% Composition Checksum; disabled: to be correctly located 
% errx = sum( xis ) - 1 ;
% % if( abs( errx ) > 1e-8 )
% if( abs( errx ) > 1e-5 )
%    fprintf( 'function %s :\n    Check failed: ( sum( xis ) - 1 ) = %8.4g > 1e-8\n' , ...
%                                                 mfilename , errx ) ;
%    split = -1 ;
%    return ;
% end
% 
% erry = sum( yis ) - 1 ;
% % if( abs( erry ) > 1e-8 )
% if( abs( erry ) > 1e-5 )
%    fprintf( 'function %s :\n    Check failed: ( sum( yis ) - 1 ) = %8.4g > 1e-8\n' , ...
%                                                 mfilename , erry ) ;
%    split = -1 ;
%    return ;
% end

split = [ V zeros( 1 , ncomp - 1 ) ; xis ; yis ] ;

return ;

end
% -------------------------------------------------------------------------------- __END__





















