function params = SRK_mix_parameters_kij( T , P , composition , props , alphafun , mixrule )
% Calculates Soave-Redlich-Kwong mixture parameters.
% Arguments: T , P = [ K , bar ]
%            composition = molar fraction mixture composition vector.
%            props = Property matrix, one column per component, containing -at least-
%                    Tc [K] , Pc [bar] and Pitzer's w [] in the first three rows.  
%            alphafun = string containing the alpha function type to use , among
%                       'orig'||'SRK' , 'Ledanois' , 'orig+BM' , 'Ledanois+BM'
%            mixrule  = string containing the mixing rule type to use, among : 'Virial'
% Returns  : params   = matrix containing SRK mixture parameters 
%                        [ Amix 0     ...    0             []
%                          Bmix 0     ...    0             []
%                          Dmix 0     ...    0             []
%                          amix 0     ...    0             [bar*L^2/mol^2]
%                          bmix 0     ...    0             [L/mol]
%                          B1   B2    ...    Bn            [] 
%                          A11  A12   ...    A1n           [] cross coefficients
%                          A21  A22   ...    A2n           [] cross coefficients
%                          ...  ...   ...    ...           [] cross coefficients
%                          An1  An2   ...    Ann  ]        [] cross coefficients

% Copyright (c) Fidel Mato & UVa 2016
% Author    : Fidel Mato <fidel@iq.uva.es>
% Maintainer: Fidel Mato <fidel@iq.uva.es>
% Keywords  : Parameters, Cubic EOS , Soave Redlich Kwong

% **********************************************************************
% BEWARE: Just checked for binary C1-C3, SRK k12 = 0.0241509 []
% **********************************************************************


R        = 0.08314459848 ;                                 % [bar*L/mol/K]

[ rows , ncomp ] = size( props ) ;                         % ncompounds

as       = [  ] ;                                          % [bar*L^2/mol^2]
ms       = [  ] ;                                          % []

Tcs      = props( 1 , : ) ;                                % [K]
Pcs      = props( 2 , : ) ;                                % [bar]
ws       = props( 3 , : ) ;                                % []

if( rows - 3 == ncomp )
    kijs = props( 4 : end , : )   ;                        % [] 1PSRK cross-coefficients
else
    kijs = zeros( ncomp , ncomp ) ;                        % [] No 1PSRK cross-coeffs.
end

acs      = 0.427480*R^2* Tcs.^2 ./ Pcs ;                   % [bar*L^2/mol^2]
bs       = 0.086640*R*Tcs ./ Pcs       ;                   % [L/mol]
Bs       = bs.*P/R/T                   ;                   % []

for( k = 1 : ncomp )   % To be vectorized
                       % alphas = arrayfun( SRK_1C_alpha( alphafun , Tcs , ws ) ) 
    Tc = Tcs( k ) ;                                        % [K]
    w  =  ws( k ) ;                                        % []
    Tr = T/Tc     ;                                        % []

    [ alpha m ] = SRK_1C_alpha( alphafun , Tr , w ) ;      % []
    as( end + 1 ) = alpha*acs( k )               ;         % [bar*L^2/mol^2]
    ms( end + 1 ) = m                            ;         % []

end

Dmix     = 0 ;                                             % [bar*L^2/mol^2]
for( k = 1 : ncomp )   % To be vectorized
    for( j = 1 : ncomp )   % To be vectorized
        
        Dmix = Dmix + ms( j )*composition( k )*composition( j )* ...
               sqrt( as( k ) )*sqrt( acs( j ) *T/Tcs( j ) ) ;          % [bar*L^2/mol^2]
        
    end
end

as       = repmat( as , ncomp , 1 ) ;                      % [bar*L^2/mol^2]
aijs     = ( as.*as' ).^0.5         ;                      % []

aijs     = ( 1 - kijs ).*aijs       ;                      % [] + cross coefficients

Aijs     = aijs*P/R^2/T^2           ;                      % [] 

amix     = repmat( composition , ncomp , 1 ) ;             % [mol.frac]
amix     = sum( sum( amix.*amix'.*aijs ) )   ;             % [bar*L^2/mol^2]
bmix     = sum( composition.*bs )            ;             % [L/mol]

Amix     = amix*P/R^2/T^2                    ;             % []
Bmix     = bmix*P/R/T                        ;             % []

params   = [ Amix zeros( 1 , ncomp - 1 )      ;            % []
             Bmix zeros( 1 , ncomp - 1 )      ;            % []
             Dmix zeros( 1 , ncomp - 1 )      ;            % []
             amix zeros( 1 , ncomp - 1 )      ;            % [bar*L^2/mol^2]
             bmix zeros( 1 , ncomp - 1 )      ;            % [L/mol]
             Bs                           ;                % []
             Aijs                           ] ;            % []

end
% -------------------------------------------------------------------------------- __END__
