function phis = SRK_parc_fug_coef_all( comp , params , Z )
% Calculates SRK partial fugacity coefficient for all components in a mixture
% Arguments: comp   = molar fraction mixture composition vector.
%            params = (see function SRK_mix_parameters( T , P , ... ) )  
%                     matrix containing SRK mixture parameters 
%                        [ Amix 0     ...    0             []
%                          Bmix 0     ...    0             []
%                          Dmix 0     ...    0             []
%                          amix 0     ...    0             [bar*L^2/mol^2]
%                          bmix 0     ...    0             [L/mol]
%                          B1   B2    ...    Bn            [] 
%                          A11  A12   ...    A1n           [] cross coefficients
%                          A21  A22   ...    A2n           [] cross coefficients
%                          ...  ...   ...    ...           [] cross coefficients
%                          An1  An2   ...    Ann  ]        [] cross coefficients
%            Z      = mixture compressibility factor       [] 
% Returns  : phis   = partial fugacity coefficients vector []

% Copyright (c) Fidel Mato & UVa 2016
% Author    : Fidel Mato <fidel@iq.uva.es>
% Maintainer: Fidel Mato <fidel@iq.uva.es>
% Keywords  : Partial fugacity coefficients, Mixture , Cubic EOS , Soave Redlich Kwong

Amix     = params( 1 , 1 ) ;                               % []
Bmix     = params( 2 , 1 ) ;                               % []

ncomp    = length( comp( 1 , : ) ) ;
phis     = zeros( ncomp , 1 )'     ;                       % []

for( k = 1 : ncomp )   % To be vectorized

    Bk        = params( 6 , k ) ;                          % []
    Akjs      = params( 6 + k , : ) ;                      % []

    phis( k ) = exp( Bk/Bmix*( Z - 1 ) - log( Z - Bmix ) + ...
                    Amix/Bmix*log( 1 + Bmix/Z ) * ...
                    ( Bk/Bmix - 2/Amix*sum( comp.*Akjs ) ) ) ;      % []

end
% -------------------------------------------------------------------------------- __END__
