function condcomp = SRK_phase_stability( T , P , zis , props , alphafun , mixrule )
% Performs a Michelsen phase stability analysis (see ref. [1] below):
% following Whitson & Brulé approach, ref. [2]:
%
% Call     : SRK_stability_analysis( T , P , zis , props , alphafun , mixrule )
% Arguments: T , P == [ K , bar ]
%            zis   == molar fraction mixture composition vector.
%            props == property matrix, one column per component, containing -at least-
%                     Tc [K] , Pc [bar] and Pitzer's w [] in the first three rows,
%                     and, optionally, as many rows as components containing cross coeffs:  
%                     k11  k12   ...    k1n           [] cross coefficients
%                     ...
%                     kn1  kn2   ...    knn           [] cross coefficients
%            alphafun == string containing the alpha function type to use , among
%                        'orig'||'SRK' , 'Ledanois' , 'orig+BM' , 'Ledanois+BM' , 'Twu'
%            mixrule  == string containing the mixing rule type to use, among : 'Virial'
% Returns  : condcomp, a 5xNumberComponents matrix, as
%                [ stable   pnvals -1 ... -1 
%                  errcode   -1       ... -1
%                    y1               ... yn    
%                    x1               ... xn    
%                    K1               ... Kn ]
%            where: stable    == { 1 | 0 | -1 } == { stable | unstable | can't be said }
%                   pnvals    == probable number of Valleys (as in ref. [3])
%                   errcode   == see Errors below
%                   y1 ... yn == vapour-like fase composition [mol frac.] or -1 ... -1
%                   x1 ... xn == liquid-like fase composition [mol frac.] or -1 ... -1
%                   K1 ... Kn == suggested K-values []                    or -1 ... -1
% Errors   :  12 == tests 1&2 converged
%             -1 == 2 converged , 1 not
%             -2 == 1 converged , 2 not
%            -12 == tests 1&2 not converged
% References:
%  [1] Michelsen, M. L. The isothermal flash problem. Part I. Stability
%      Fluid Phase Equilibria 1982,9,1-19, doi:10.1016/0378-3812(82)85001-2
%  [2] Curtis H. Whitson and Michael R. Brule (2000)
%      Phase Behavior, SPE Monograph Series Vol. 20 , ISBN: 978-1-55563-087-4
%  [3] Baker, L. E., Pierce, A. C., & Luks, K. D. (1982, October 1).
%      Gibbs Energy Analysis of Phase Equilibria. Society of Petroleum Engineers.
%      doi:10.2118/9806-PA
%
% Copyright (c) Fidel Mato & UVa 2018

% Author    : Fidel Mato <fidel@iq.uva.es>
% Maintainer: Fidel Mato <fidel@iq.uva.es>
% Keywords  : Stability , Phases , Flash , K-values , Michelsen , Whitson , Rachford-Rice

% TODO: Try some cases where (   nV   > 1 ) && (   nL   > 1 )
%       in order to check (KisV./KisL) == (yis./xis)


% ========================================================================= Specifications
maxiter  = 1000            ;
convgtol = 1e-12           ;                               % convergence tolerance
trivltol = 1e-4            ;                               % trivial solution tolerance
ss2gdem  = 4               ;                               % steps of S.S. per GDEM step

% ---------------------------------------------------------- Initializations
nV       = -1 ; nL    = -1 ;                               % [mol] phase mol number
conv1    = -1 ; conv2 = -1 ;                               % converged flags
triv1    = -1 ; triv2 = -1 ;                               % trivial solution flags
stable   = -1              ;                               % stable/unstable flag
ncomp    = length( zis )   ;                               % number of components
errcode  = 0               ;                               % error condition

% ---------------------------------------------------------- Nested fuction definition
function mingZ = mingZ( comp , params , Zs )
% Between multiple (2) Z-factor roots, choose the root with the lowest g*
% ( g* = G/R/T [] , Normalized Gibbs Free Energy )
    if( ~isreal( Zs( 1 ) ) )
        mingZ = Zs( 2 ) ;                                  % []
    elseif( ~isreal( Zs( 2 ) ) )
        mingZ = Zs( 1 ) ;                                  % []
    else % ----------------- both real
        phis1 = SRK_parc_fug_coef_all( comp , params , Zs( 1 ) )                ; % []
        phis2 = SRK_parc_fug_coef_all( comp , params , Zs( 2 ) )                ; % []
        % g1 = sum( zis.*log( phis1*P ) ) or g1 = sum( zis.*log( phis1 ) ) ? % G/R/T
        [ x ix ] = min( [ sum( zis.*log( phis1 ) ) sum( zis.*log( phis2 ) ) ] ) ;
        mingZ = Zs( ix ) ;                                 % []
    end
    return ;
end % ------------------------------------------------------ mingZ( )

% =========================================================================== Calculations
params   = SRK_mix_parameters_kij( T , P , zis , props , alphafun , mixrule ) ;
Zs       = SRK_compress_factors( params )    ;             % []
Zmix     = mingZ( zis , params , Zs )        ;             % [] lowest g* Z-factor root

Kmixs    = Wilson_correl_Ks( T , P , props ) ;             % [] Initial estimation
fugimixs = zis.*SRK_parc_fug_coef_all( zis , params , Zmix ) ; % [bar]

% ----------------------------------------- test 1: create a second, new vapour-like phase

gdemcnt  = 0                                 ;             % S.S. steps counter for GDEM
Kis      = Kmixs                             ;             % [] Initial estimation
for( k = 1 : maxiter )
    
    niVs   = zis.*Kis                        ;             % [mol]
    nV     = sum( niVs )                     ;             % [mol]
    yis    = niVs/nV                         ;             % [mol.frac] normalize

    parms  = SRK_mix_parameters_kij( T , P , yis , props , alphafun , mixrule ) ;
    Zs     = SRK_compress_factors( parms )   ;             % []
    Z      = mingZ( yis , parms , Zs )       ;             % [] lowest g* Z root

    fugiVs = yis.*SRK_parc_fug_coef_all( yis , parms , Z ) ; % []
    Ris    = fugimixs./fugiVs/nV             ;             % [1/mol] ???
    if( sum( ( Ris - 1 ).^2 ) <= convgtol )
        conv1 = 1 ; triv1 = 0 ;  break       ;
    end

    gdemcnt = gdemcnt + 1                               ;
    if( gdemcnt > ss2gdem ) % --------------------------- GDEM promotion
        b01     = sum( log( Ris ).*log( Risprev ) )     ;
        b11     = sum( log( Risprev ).*log( Risprev ) ) ;
        Kis     = Kis.*Ris.^( b11/( b11 - b01 ) )       ;
        gdemcnt = 0                                     ;
    end
    
    Risprev = Ris      ; 
    Kisnew  = Ris.*Kis ;                                   % []
    if( sum( ( log( Kisnew ) ).^2 ) <= trivltol )
        conv1 = 1 ; triv1 = 1 ; break ;
    end
    Kis     = Kisnew   ;
    
end
if( k >= maxiter)
    conv1 = 0  ;                                           % not converged
end
KisV     = Kis ;                                           % []

% ----------------------------------------- test 2: create a second, new liquid-like phase

gdemcnt  = 0                                 ;             % S.S. steps counter for GDEM
Kis      = Kmixs                             ;             % [] Initial estimation
for( k = 1 : maxiter )

    niLs   = zis./Kis                        ;             % [mol]
    nL     = sum( niLs )                     ;             % [mol]
    xis    = niLs/nL                         ;             % [mol.frac] normalize

    parms = SRK_mix_parameters_kij( T , P , xis , props , alphafun , mixrule ) ;
    Zs    = SRK_compress_factors( parms )    ;             % []
    Z     = mingZ( xis , parms , Zs )        ;             % [] lowest g* Z root

    fugiLs = xis.*SRK_parc_fug_coef_all( xis , parms , Z ) ; % []
    Ris    = fugiLs./fugimixs*nL             ;             % [mol] ???
    if( sum( ( Ris - 1 ).^2 ) <= convgtol )
        conv2 = 1 ; triv2 = 0 ;  break       ;
    end

    gdemcnt = gdemcnt + 1                               ;
    if( gdemcnt > ss2gdem ) % --------------------------- GDEM promotion
        b01     = sum( log( Ris ).*log( Risprev ) )     ;
        b11     = sum( log( Risprev ).*log( Risprev ) ) ;
        Kis     = Kis.*Ris.^( b11/( b11 - b01 ) )       ;
        gdemcnt = 0                                     ;
    end
    
    Risprev = Ris ; 
    Kisnew = Ris.*Kis                        ;             % []
    if( sum( ( log( Kisnew ) ).^2 ) <= trivltol )
        conv2 = 1 ; triv2 = 1 ; break        ;
    end
    
    Kis      = Kisnew ;
    
end
if( k >= maxiter)
    conv2 = 0  ;                                           % not converged
end
KisL     = Kis ;                                           % []

% ================================================================================ Results
% The mixture is stable (single-phase condition prevails) if:
%     - Both test yield n <= 1 ( nV <=1 and nL <= 1 )
%     - Or both test converge to trivial solution
%     - Or one test converges to the trivial solution and the other gives n <= 1
%
% Only one test indicating n > 1 is sufficient to determine that the
% mixture is unstable and that the two-phase condition prevails. The same
% conclusion is made if both test give n > 1 , or if one of the tests
% converges to the trivial solution and the other gives n > 1 .

minus1s  =  -1*ones( 1, ncomp )                 ;
if( conv1 == 1 && conv2 == 1 )

    errcode  = 12                               ;          % tests 1&2 converged
    if(     ( triv1 == 1 ) && ( triv2 == 1 ) ) 
        stable = 1 ; pnvals = 1 ; Kis = minus1s ;
    elseif( (   nL  <= 1 ) && ( triv2 == 1 ) )
        stable = 1 ; pnvals = 2 ; Kis = minus1s ;
    elseif( ( triv1 == 1 ) && (   nL  <= 1 ) )
        stable = 1 ; pnvals = 2 ; Kis = minus1s ;
    elseif( (   nV  <= 1 ) && (   nL  <= 1 ) )
        stable = 1 ; pnvals = 3 ; Kis = minus1s ;
    elseif( (   nV   > 1 ) && ( triv2 == 1 ) )
        stable = 0 ; pnvals = 2 ; Kis = KisV    ;
    elseif( ( triv1 == 1 ) && (   nL   > 1 ) )
        stable = 0 ; pnvals = 2 ; Kis = KisL    ;
    elseif( (   nV   > 1 ) && (   nL   > 1 ) )
        stable = 0 ; pnvals = 2 ; Kis = KisV./KisL ;
    elseif( (   nV   > 1 ) && (   nL  <= 1 ) )
        stable = 0 ; pnvals = 3 ; Kis = KisV    ;
    elseif( (   nV  <= 1 ) && (   nL   > 1 ) )
        stable = 0 ; pnvals = 3 ; Kis = KisL    ;
    end
    condcomp = [ stable pnvals -1*ones( 1 , ncomp - 2 ) ]           ;
    condcomp = [ condcomp ; [ errcode  -1*ones( 1 , ncomp - 1 ) ] ] ;
%    condcomp = [ condcomp ; yis ; xis ; Kis ]                       ;
    condcomp = [ condcomp ; yis ; xis ; KisV ; KisL ]                       ; % trying

elseif( conv1 == 1 && conv2 ~= 1 )

    condcomp = [ minus1s ; minus1s ; yis ; minus1s ; KisV ] ;
    errcode  = -2 ;                
    condcomp( 2 , 1 ) = -2  ;                              % errcode: 1 converged , 2 not

elseif( conv1 ~= 1 && conv2 == 1 )

    condcomp = [ minus1s ; minus1s ; minus1s ; xis ; KisL ] ;
    condcomp( 2 , 1 ) = -1  ;                              % errcode: 2 converged , 1 not

elseif( conv1 ~= 1 && conv2 ~= 1 )

    condcomp = [ minus1s ; minus1s ; minus1s ; minus1s ; minus1s ] ;
    condcomp( 2 , 1 ) = -12 ;                              % errcode: 1&2 not converged

end

return ;
end
% -------------------------------------------------------------------------------- __END__









