function Psat= PR_1C_Psat( T , Tc , Pc , w )

% Calculates saturation P for a T using the Peng-Robinson EoS
% Arguments: ( T , Tc , Pc , w ) = [ K , K , bar , dimless ]
% Returns  : Psat [ bar ] or error
% Errors   : -1 => complex roots found by solver
%          : -2 => supercritical conditions

% Copyright (C) 2016 Fidel Mato
% Author: Fidel Mato <fidel@iq.uva.es>
% Keywords: saturation pressure, vapor pressure , EOS , Peng-Robinson
% Maintainer: Fidel Mato <fidel@iq.uva.es>


R          = 0.08314459848 ;                               % [bar*L/mol/K]
precission = 1e-6 ;                                        % [dimless]

Tr       = T/Tc                 ;                          % [dimless]

% Checking supercritical conditions:
if( T > Tc )
    Psat = -2 ;
    return    ;
end

ac       = 0.45724*R^2*Tc^2/Pc   ;                         % [bar*L^2/mol^2] SRK
b        = 0.07780*R*Tc/Pc       ;                         % [L/mol]         SRK
alpha    = PR_1C_alpha( Tr , w ) ;                         % [dimless]
a        = ac*alpha              ;                         % [bar*L^2/mol^2]

% ---------------------------------------------- P estimation ( from PR 1st derivative)
% R*T*V^4 + (4*b*R*T - 2*a)*V^3 + (2*b^2*R*T + 2*a*b)*V^2 + ...
%                             + (2*a*b^2 - 4*b^3*R*T)*V + b^4*R*T - 2*a*b^3 == 0
coefs = [ R*T , ( 4*b*R*T - 2*a ) , ( 2*b^2*R*T + 2*a*b ) ] ;
coefs = [ coefs , ( 2*a*b^2 - 4*b^3*R*T ) , ( b^4*R*T - 2*a*b^3 ) ] ; 
Vmax  = max( roots( coefs ) ) ;                            % [L/mol]
Pmax  = R*T/( Vmax - b ) - a/( Vmax*( Vmax + b ) + b*( Vmax - b ) ) ; % [bar]
P     = Pmax - 1e-6;                                       % [bar]

while( true )

    [ Zv Zl ] = PR_1C_compress_factors( T , P , Tc , Pc , w ) ; % [dimless]
    if ~( isreal( Zv ) && isreal( Zl  ))                   % if any not real...
        fprintf('\nFunction %s error: Complex root\n\n' , mfilename() ); % explain...
        Psat = -1 ;   
        return    ;
    end
    Pr    = P/Pc ;                                         % [dimless]
    A     = 0.45724*alpha*Pr/Tr^2  ;                       % [dimless]
    B     = 0.07780*Pr/Tr          ;                       % [dimless]
    phiv = exp( Zv - 1 - log( Zv - B ) - ...
              A/( 2*sqrt( 2 )*B )*log( ( Zv + 2.414*B )/( Zv - 0.414*B ) ) ) ; % [dimless]
    phil = exp( Zl - 1 - log( Zl - B ) - ...
              A/( 2*sqrt( 2 )*B )*log( ( Zl + 2.414*B )/( Zl - 0.414*B ) ) ) ; % [dimless]
    if( abs( phiv - phil ) < precission )
        break ;
    end
    P = P*phil/phiv ;                                      % [bar]

end
Psat = P ;                                                 % [bar]

end
% -------------------------------------------------------------------------------- __END__
