function Psat = SRK_1C_Psat( T , Tc , Pc , w , alpha_function )

% Calculates saturation P for a T using the Soave-Redlich-Kwong EoS
% Arguments: ( T , Tc , Pc , w , alpha_function ) = [ K , K , bar , dimless , string ]
%              alpha_function: string defining the alpha function type to use, among
%                             'orig'||'SRK' , 'Ledanois' , 'orig+BM' , 'Ledanois+BM'
%                             [ See : SRK_1C_alpha( alpha_function , Tr , w ) ]
% Returns  : Psat [ bar ] or error
% Errors   : -1 => complex roots found by solver
%          : -2 => supercritical conditions

% Copyright (C) 2016 Fidel Mato
% Author: Fidel Mato <fidel@iq.uva.es>
% Keywords: saturation pressure, vapor pressure , EOS , Soave Redlich Kwong
% Maintainer: Fidel Mato <fidel@iq.uva.es>


R          = 0.08314459848 ;                               % [bar*L/mol/K]
precission = 1e-6 ;                                        % [dimless]

Tr       = T/Tc                 ;                          % [dimless]

% Checking supercritical conditions:
if( T > Tc )
    Psat = -2 ;
    return    ;
end

ac          = 0.427480*R^2*Tc^2/Pc   ;                     % [bar*L^2/mol^2] SRK
b           = 0.086640*R*Tc/Pc       ;                     % [L/mol]         SRK
[ alpha m ] = SRK_1C_alpha( alpha_function , Tr , w ) ;    % [dimless]
a           = ac*alpha               ;                     % [bar*L^2/mol^2]

% ---------------------------------------------- P estimation ( from SRK 1st derivative)
% V^4 + (2*b - 2*a*alfa/R/T)*V^3 + (b^2 + 3*a*alfa*b/R/T)*V^2 - a*alfa*b^3/R/T == 0
coefs = [ R*T , +(2*b*R*T - 2*a) , +(b^2*R*T + 3*a*b) , 0 , -a*b^3 ]   ;        
Vmax  = max( roots( coefs ) ) ;                            % [L/mol]
Pmax  = R*T/( Vmax - b ) - a/Vmax/( Vmax + b ) ;           % [bar]
P     = Pmax - 1e-6;                                       % [bar]

while( 1 == 1 )

    [ Zv Zl ] = SRK_1C_compress_factors( T , P , Tc , Pc , w , alpha_function ) ; % [dimless]
    if ~( isreal( Zv ) && isreal( Zl  ))                   % if any not real...
        fprintf('\nFunction %s error: Complex root\n\n' , mfilename() ); % explain...
        Psat = -1 ;   
        return    ;
    end
    Pr    = P/Pc                   ;                       % [dimless]
    A     = 0.42748*alpha*Pr/Tr^2  ;                       % [dimless]
    B     = 0.08664*Pr/Tr          ;                       % [dimless]
    phiv = exp( Zv - 1 - log( Zv - B ) - A/B*log( 1 + B/Zv ) ) ; % [dimless]
    phil = exp( Zl - 1 - log( Zl - B ) - A/B*log( 1 + B/Zl ) ) ; % [dimless]
    if( abs( phiv - phil ) < precission )
        break ;
    end
    P = P*phil/phiv ;                                      % [bar]

end
Psat = P ;                                                 % [bar]

end
% -------------------------------------------------------------------------------- __END__
